@@ -37,11 +37,6 @@
 
 struct t_hashtable *weechat_hdata = NULL;
 
-/* hashtables used in hdata_search() for evaluating expression */
-struct t_hashtable *hdata_search_pointers = NULL;
-struct t_hashtable *hdata_search_extra_vars = NULL;
-struct t_hashtable *hdata_search_options = NULL;
-
 char *hdata_type_string[9] =
 { "other", "char", "integer", "long", "string", "pointer", "time",
   "hashtable", "shared_string" };
@@ -52,8 +47,7 @@ char *hdata_type_string[9] =
  */
 
 void
-hdata_free_var (struct t_hashtable *hashtable,
-                const void *key, void *value)
+hdata_free_var_cb (struct t_hashtable *hashtable, const void *key, void *value)
 {
     struct t_hdata_var *var;
 
@@ -77,8 +71,7 @@ hdata_free_var (struct t_hashtable *hashtable,
  */
 
 void
-hdata_free_list (struct t_hashtable *hashtable,
-                 const void *key, void *value)
+hdata_free_list_cb (struct t_hashtable *hashtable, const void *key, void *value)
 {
     /* make C compiler happy */
     (void) hashtable;
@@ -120,13 +113,13 @@ hdata_new (struct t_weechat_plugin *plugin, const char *hdata_name,
                                              WEECHAT_HASHTABLE_POINTER,
                                              NULL,
                                              NULL);
-        new_hdata->hash_var->callback_free_value = &hdata_free_var;
+        new_hdata->hash_var->callback_free_value = &hdata_free_var_cb;
         new_hdata->hash_list = hashtable_new (32,
                                               WEECHAT_HASHTABLE_STRING,
                                               WEECHAT_HASHTABLE_POINTER,
                                               NULL,
                                               NULL);
-        new_hdata->hash_list->callback_free_value = &hdata_free_list;
+        new_hdata->hash_list->callback_free_value = &hdata_free_list_cb;
         hashtable_set (weechat_hdata, hdata_name, new_hdata);
         new_hdata->create_allowed = create_allowed;
         new_hdata->delete_allowed = delete_allowed;
@@ -148,6 +141,7 @@ hdata_new_var (struct t_hdata *hdata, const char *name, int offset, int type,
                const char *hdata_name)
 {
     struct t_hdata_var *var;
+    const char *ptr_array_size;
 
     if (!hdata || !name)
         return;
@@ -158,8 +152,20 @@ hdata_new_var (struct t_hdata *hdata, const char *name, int offset, int type,
         var->offset = offset;
         var->type = type;
         var->update_allowed = update_allowed;
-        var->array_size = (array_size && array_size[0]) ? strdup (array_size) : NULL;
-        var->hdata_name = (hdata_name && hdata_name[0]) ? strdup (hdata_name) : NULL;
+        ptr_array_size = array_size;
+        if (ptr_array_size && (strncmp (ptr_array_size, "*,", 2) == 0))
+        {
+            var->array_pointer = 1;
+            ptr_array_size += 2;
+        }
+        else
+        {
+            var->array_pointer = 0;
+        }
+        var->array_size = (ptr_array_size && ptr_array_size[0]) ?
+            strdup (ptr_array_size) : NULL;
+        var->hdata_name = (hdata_name && hdata_name[0]) ?
+            strdup (hdata_name) : NULL;
         hashtable_set (hdata->hash_var, name, var);
     }
 }
@@ -394,7 +400,7 @@ hdata_get_var (struct t_hdata *hdata, void *pointer, const char *name)
 {
     int offset;
 
-    if (!hdata || !pointer)
+    if (!hdata || !pointer || !name)
         return NULL;
 
     offset = hdata_get_var_offset (hdata, name);
@@ -576,72 +582,78 @@ hdata_move (struct t_hdata *hdata, void *pointer, int count)
  */
 
 void *
-hdata_search (struct t_hdata *hdata, void *pointer, const char *search, int move)
+hdata_search (struct t_hdata *hdata,
+              void *pointer,
+              const char *search,
+              struct t_hashtable *pointers,
+              struct t_hashtable *extra_vars,
+              struct t_hashtable *options,
+              int move)
 {
+    struct t_hashtable *pointers2, *options2;
     char *result;
+    void *ret_pointer;
     int rc;
 
     if (!hdata || !pointer || !search || !search[0] || (move == 0))
         return NULL;
 
-    /* clear or create hashtable with pointer for search */
-    if (hdata_search_pointers)
+    ret_pointer = NULL;
+
+    /* duplicate or create hashtable with pointers */
+    if (pointers)
     {
-        hashtable_remove_all (hdata_search_pointers);
+        pointers2 = hashtable_dup (pointers);
     }
     else
     {
-        hdata_search_pointers = hashtable_new (32,
-                                               WEECHAT_HASHTABLE_STRING,
-                                               WEECHAT_HASHTABLE_POINTER,
-                                               NULL,
-                                               NULL);
+        pointers2 = hashtable_new (32,
+                                   WEECHAT_HASHTABLE_STRING,
+                                   WEECHAT_HASHTABLE_POINTER,
+                                   NULL,
+                                   NULL);
     }
 
-    /*
-     * create hashtable with extra vars (empty hashtable)
-     * (hashtable would be created in eval_expression(), but it's created here
-     * so it will not be created for each call to eval_expression())
-     */
-    if (!hdata_search_extra_vars)
+    /* duplicate or create hashtable with options */
+    if (options)
     {
-        hdata_search_extra_vars = hashtable_new (32,
-                                                 WEECHAT_HASHTABLE_STRING,
-                                                 WEECHAT_HASHTABLE_STRING,
-                                                 NULL,
-                                                 NULL);
+        options2 = hashtable_dup (options);
     }
-
-    if (!hdata_search_options)
+    else
     {
-        hdata_search_options = hashtable_new (32,
-                                              WEECHAT_HASHTABLE_STRING,
-                                              WEECHAT_HASHTABLE_STRING,
-                                              NULL,
-                                              NULL);
-        if (hdata_search_options)
-            hashtable_set (hdata_search_options, "type", "condition");
+        options2 = hashtable_new (32,
+                                  WEECHAT_HASHTABLE_STRING,
+                                  WEECHAT_HASHTABLE_POINTER,
+                                  NULL,
+                                  NULL);
     }
+    if (options2)
+        hashtable_set (options2, "type", "condition");
 
     while (pointer)
     {
         /* set pointer in hashtable (used for evaluating expression) */
-        hashtable_set (hdata_search_pointers, hdata->name, pointer);
+        if (pointers2)
+            hashtable_set (pointers2, hdata->name, pointer);
 
         /* evaluate expression */
-        result = eval_expression (search, hdata_search_pointers,
-                                  hdata_search_extra_vars,
-                                  hdata_search_options);
+        result = eval_expression (search, pointers2, extra_vars, options2);
         rc = eval_is_true (result);
         if (result)
             free (result);
         if (rc)
-            return pointer;
+        {
+            ret_pointer = pointer;
+            goto end;
+        }
 
         pointer = hdata_move (hdata, pointer, move);
     }
 
-    return NULL;
+end:
+    hashtable_free (pointers2);
+    hashtable_free (options2);
+    return ret_pointer;
 }
 
 /*
@@ -706,9 +718,16 @@ hdata_char (struct t_hdata *hdata, void *pointer, const char *name)
     if (var && (var->offset >= 0))
     {
         if (var->array_size && (index >= 0))
-            return (*((char **)(pointer + var->offset)))[index];
+        {
+            if (var->array_pointer)
+                return (*((char **)(pointer + var->offset)))[index];
+            else
+                return ((char *)(pointer + var->offset))[index];
+        }
         else
+        {
             return *((char *)(pointer + var->offset));
+        }
     }
 
     return '\0';
@@ -733,9 +752,16 @@ hdata_integer (struct t_hdata *hdata, void *pointer, const char *name)
     if (var && (var->offset >= 0))
     {
         if (var->array_size && (index >= 0))
-            return ((int *)(pointer + var->offset))[index];
+        {
+            if (var->array_pointer)
+                return (*((int **)(pointer + var->offset)))[index];
+            else
+                return ((int *)(pointer + var->offset))[index];
+        }
         else
+        {
             return *((int *)(pointer + var->offset));
+        }
     }
 
     return 0;
@@ -760,9 +786,16 @@ hdata_long (struct t_hdata *hdata, void *pointer, const char *name)
     if (var && (var->offset >= 0))
     {
         if (var->array_size && (index >= 0))
-            return ((long *)(pointer + var->offset))[index];
+        {
+            if (var->array_pointer)
+                return (*((long **)(pointer + var->offset)))[index];
+            else
+                return ((long *)(pointer + var->offset))[index];
+        }
         else
+        {
             return *((long *)(pointer + var->offset));
+        }
     }
 
     return 0;
@@ -787,9 +820,19 @@ hdata_string (struct t_hdata *hdata, void *pointer, const char *name)
     if (var && (var->offset >= 0))
     {
         if (var->array_size && (index >= 0))
-            return (*((char ***)(pointer + var->offset)))[index];
+        {
+            if (var->array_pointer)
+                return (*((char ***)(pointer + var->offset)))[index];
+            else
+            {
+                /* we can not index a static array of strings */
+                return NULL;
+            }
+        }
         else
+        {
             return *((char **)(pointer + var->offset));
+        }
     }
 
     return NULL;
@@ -814,9 +857,16 @@ hdata_pointer (struct t_hdata *hdata, void *pointer, const char *name)
     if (var && (var->offset >= 0))
     {
         if (var->array_size && (index >= 0))
-            return (*((void ***)(pointer + var->offset)))[index];
+        {
+            if (var->array_pointer)
+                return (*((void ***)(pointer + var->offset)))[index];
+            else
+                return ((void **)(pointer + var->offset))[index];
+        }
         else
+        {
             return *((void **)(pointer + var->offset));
+        }
     }
 
     return NULL;
@@ -841,9 +891,16 @@ hdata_time (struct t_hdata *hdata, void *pointer, const char *name)
     if (var && (var->offset >= 0))
     {
         if (var->array_size && (index >= 0))
-            return ((time_t *)(pointer + var->offset))[index];
+        {
+            if (var->array_pointer)
+                return (*((time_t **)(pointer + var->offset)))[index];
+            else
+                return ((time_t *)(pointer + var->offset))[index];
+        }
         else
+        {
             return *((time_t *)(pointer + var->offset));
+        }
     }
 
     return 0;
@@ -868,9 +925,16 @@ hdata_hashtable (struct t_hdata *hdata, void *pointer, const char *name)
     if (var && (var->offset >= 0))
     {
         if (var->array_size && (index >= 0))
-            return (*((struct t_hashtable ***)(pointer + var->offset)))[index];
+        {
+            if (var->array_pointer)
+                return (*((struct t_hashtable ***)(pointer + var->offset)))[index];
+            else
+                return ((struct t_hashtable **)(pointer + var->offset))[index];
+        }
         else
+        {
             return *((struct t_hashtable **)(pointer + var->offset));
+        }
     }
 
     return NULL;
@@ -898,6 +962,9 @@ hdata_compare (struct t_hdata *hdata, void *pointer1, void *pointer2,
     void *ptr_value1, *ptr_value2;
     time_t time_value1, time_value2;
 
+    if (!hdata || !name)
+        return 0;
+
     if (!pointer1 && pointer2)
         return -1;
     if (pointer1 && !pointer2)
@@ -1048,17 +1115,25 @@ hdata_set (struct t_hdata *hdata, void *pointer, const char *name,
             return 1;
             break;
         case WEECHAT_HDATA_POINTER:
-            rc = sscanf (value, "%lx", &ptr);
-            if ((rc != EOF) && (rc != 0))
+            if (value)
             {
-                *((void **)(pointer + var->offset)) = (void *)ptr;
+                rc = sscanf (value, "%lx", &ptr);
+                if ((rc != EOF) && (rc != 0))
+                {
+                    *((void **)(pointer + var->offset)) = (void *)ptr;
+                    return 1;
+                }
+            }
+            else
+            {
+                *((void **)(pointer + var->offset)) = NULL;
                 return 1;
             }
             break;
         case WEECHAT_HDATA_TIME:
             error = NULL;
             number = strtol (value, &error, 10);
-            if (error && !error[0])
+            if (error && !error[0] && (number >= 0))
             {
                 *((time_t *)(pointer + var->offset)) = (time_t)number;
                 return 1;
@@ -1107,8 +1182,6 @@ hdata_update (struct t_hdata *hdata, void *pointer,
     value = hashtable_get (hashtable, "__update_allowed");
     if (value)
     {
-        if (!hdata->callback_update)
-            return 0;
         var = hashtable_get (hdata->hash_var, value);
         if (!var)
             return 0;
@@ -1191,10 +1264,7 @@ hdata_free_all_plugin_map_cb (void *data, struct t_hashtable *hashtable,
     ptr_hdata = (struct t_hdata *)value;
 
     if (ptr_hdata->plugin == (struct t_weechat_plugin *)data)
-    {
-        hdata_free (ptr_hdata);
         hashtable_remove (hashtable, key);
-    }
 }
 
 /*
@@ -1207,25 +1277,6 @@ hdata_free_all_plugin (struct t_weechat_plugin *plugin)
     hashtable_map (weechat_hdata, &hdata_free_all_plugin_map_cb, plugin);
 }
 
-/*
- * Frees hdata (callback called for each hdata in memory).
- */
-
-void
-hdata_free_all_map_cb (void *data, struct t_hashtable *hashtable,
-                       const void *key, const void *value)
-{
-    struct t_hdata *ptr_hdata;
-
-    /* make C compiler happy */
-    (void) data;
-
-    ptr_hdata = (struct t_hdata *)value;
-
-    hdata_free (ptr_hdata);
-    hashtable_remove (hashtable, key);
-}
-
 /*
  * Frees all hdata.
  */
@@ -1233,7 +1284,7 @@ hdata_free_all_map_cb (void *data, struct t_hashtable *hashtable,
 void
 hdata_free_all ()
 {
-    hashtable_map (weechat_hdata, &hdata_free_all_map_cb, NULL);
+    hashtable_remove_all (weechat_hdata);
 }
 
 /*
@@ -1309,6 +1360,21 @@ hdata_print_log ()
     hashtable_map (weechat_hdata, &hdata_print_log_map_cb, NULL);
 }
 
+/*
+ * Frees a hdata in hashtable "weechat_hdata".
+ */
+
+void
+hdata_free_hdata_cb (struct t_hashtable *hashtable,
+                     const void *key, void *value)
+{
+    /* make C compiler happy */
+    (void) hashtable;
+    (void) key;
+
+    hdata_free (value);
+}
+
 /*
  * Initializes hdata: creates a hashtable with hdata.
  */
@@ -1321,6 +1387,7 @@ hdata_init ()
                                    WEECHAT_HASHTABLE_POINTER,
                                    NULL,
                                    NULL);
+    weechat_hdata->callback_free_value = &hdata_free_hdata_cb;
 }
 
 /*
@@ -1333,20 +1400,4 @@ hdata_end ()
     hdata_free_all ();
     hashtable_free (weechat_hdata);
     weechat_hdata = NULL;
-
-    if (hdata_search_pointers)
-    {
-        hashtable_free (hdata_search_pointers);
-        hdata_search_pointers = NULL;
-    }
-    if (hdata_search_extra_vars)
-    {
-        hashtable_free (hdata_search_extra_vars);
-        hdata_search_extra_vars = NULL;
-    }
-    if (hdata_search_options)
-    {
-        hashtable_free (hdata_search_options);
-        hdata_search_options = NULL;
-    }
 }
