@@ -1,7 +1,7 @@
 /*
  * gui-window.c - window functions (used by all GUI)
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -139,9 +139,9 @@ gui_window_get_context_at_xy (struct t_gui_window *window,
                               char **beginning,
                               char **end)
 {
-    int win_x, win_y;
-    char *ptr_data, *data_next_line, *str_temp;
-    char *word_start, *word_end, *last_space;
+    int win_x, win_y, coords_x_message;
+    char *data_next_line, *str_temp;
+    const char *ptr_data, *word_start, *word_end, *last_space;
 
     *chat = 0;
     *line = NULL;
@@ -183,7 +183,13 @@ gui_window_get_context_at_xy (struct t_gui_window *window,
     if (!window->coords[win_y].data)
         return;
 
-    if (win_x < window->coords_x_message)
+    coords_x_message = gui_line_get_align (
+        (*line)->data->buffer,
+        *line,
+        1,  /* with suffix */
+        ((win_y > 0) && (window->coords[win_y - 1].line != *line)));
+
+    if (win_x < coords_x_message)
     {
         /* X is before message (time/buffer/prefix) */
         if ((win_x >= window->coords[win_y].time_x1)
@@ -209,7 +215,7 @@ gui_window_get_context_at_xy (struct t_gui_window *window,
                           && (window->coords[win_y + 1].line == *line)) ?
             window->coords[win_y + 1].data : NULL;
         ptr_data = gui_chat_string_add_offset_screen (window->coords[win_y].data,
-                                                      win_x - window->coords_x_message);
+                                                      win_x - coords_x_message);
         if (ptr_data && ptr_data[0]
             && (!data_next_line || (ptr_data < data_next_line)))
         {
@@ -332,15 +338,37 @@ gui_window_tree_node_to_leaf (struct t_gui_window_tree *node,
 void
 gui_window_tree_free (struct t_gui_window_tree **tree)
 {
-    if (*tree)
+    if (!tree || !*tree)
+        return;
+
+    if ((*tree)->child1)
+        gui_window_tree_free (&((*tree)->child1));
+    if ((*tree)->child2)
+        gui_window_tree_free (&((*tree)->child2));
+
+    free (*tree);
+    *tree = NULL;
+}
+
+/*
+ * Searches a parent window which is split on the given direction
+ * ('h' or 'v').
+ */
+
+struct t_gui_window_tree *
+gui_window_tree_get_split (struct t_gui_window_tree *tree,
+                           char direction)
+{
+    if (!tree->parent_node)
+        return tree;
+
+    if (((tree->parent_node->split_horizontal) && (direction == 'h'))
+        || (!(tree->parent_node->split_horizontal) && (direction == 'v')))
     {
-        if ((*tree)->child1)
-            gui_window_tree_free (&((*tree)->child1));
-        if ((*tree)->child2)
-            gui_window_tree_free (&((*tree)->child2));
-        free (*tree);
-        *tree = NULL;
+        return tree;
     }
+
+    return gui_window_tree_get_split (tree->parent_node, direction);
 }
 
 /*
@@ -384,6 +412,7 @@ gui_window_scroll_init (struct t_gui_window_scroll *window_scroll,
     window_scroll->scrolling = 0;
     window_scroll->start_col = 0;
     window_scroll->lines_after = 0;
+    window_scroll->text_search_start_line = NULL;
     window_scroll->prev_scroll = NULL;
     window_scroll->next_scroll = NULL;
 }
@@ -396,6 +425,9 @@ void
 gui_window_scroll_free (struct t_gui_window *window,
                         struct t_gui_window_scroll *scroll)
 {
+    if (!window || !scroll)
+        return;
+
     if (scroll->prev_scroll)
         (scroll->prev_scroll)->next_scroll = scroll->next_scroll;
     if (scroll->next_scroll)
@@ -443,7 +475,8 @@ gui_window_scroll_remove_not_scrolled (struct t_gui_window *window)
                 && (ptr_scroll->start_line_pos == 0)
                 && (ptr_scroll->scrolling == 0)
                 && (ptr_scroll->start_col == 0)
-                && (ptr_scroll->lines_after == 0))
+                && (ptr_scroll->lines_after == 0)
+                && (ptr_scroll->text_search_start_line == NULL))
             {
                 gui_window_scroll_free (window, ptr_scroll);
             }
@@ -463,41 +496,41 @@ gui_window_scroll_switch (struct t_gui_window *window,
 {
     struct t_gui_window_scroll *ptr_scroll, *new_scroll;
 
-    if (window && buffer)
+    if (!window || !buffer)
+        return;
+
+    ptr_scroll = gui_window_scroll_search (window, buffer);
+
+    /* scroll is already selected (first in list)? */
+    if (ptr_scroll && (ptr_scroll == window->scroll))
+        return;
+
+    if (ptr_scroll)
     {
-        ptr_scroll = gui_window_scroll_search (window, buffer);
-
-        /* scroll is already selected (first in list)? */
-        if (ptr_scroll && (ptr_scroll == window->scroll))
-            return;
-
-        if (ptr_scroll)
-        {
-            /* scroll found, move it in first position */
-            if (ptr_scroll->prev_scroll)
-                (ptr_scroll->prev_scroll)->next_scroll = ptr_scroll->next_scroll;
-            if (ptr_scroll->next_scroll)
-                (ptr_scroll->next_scroll)->prev_scroll = ptr_scroll->prev_scroll;
-            (window->scroll)->prev_scroll = ptr_scroll;
-            ptr_scroll->prev_scroll = NULL;
-            ptr_scroll->next_scroll = window->scroll;
-            window->scroll = ptr_scroll;
-        }
-        else
-        {
-            /* scroll not found, create one and add it at first position */
-            new_scroll = malloc (sizeof (*new_scroll));
-            if (new_scroll)
-            {
-                gui_window_scroll_init (new_scroll, buffer);
-                new_scroll->next_scroll = window->scroll;
-                (window->scroll)->prev_scroll = new_scroll;
-                window->scroll = new_scroll;
-            }
-        }
-
-        gui_window_scroll_remove_not_scrolled (window);
+        /* scroll found, move it in first position */
+        if (ptr_scroll->prev_scroll)
+            (ptr_scroll->prev_scroll)->next_scroll = ptr_scroll->next_scroll;
+        if (ptr_scroll->next_scroll)
+            (ptr_scroll->next_scroll)->prev_scroll = ptr_scroll->prev_scroll;
+        (window->scroll)->prev_scroll = ptr_scroll;
+        ptr_scroll->prev_scroll = NULL;
+        ptr_scroll->next_scroll = window->scroll;
+        window->scroll = ptr_scroll;
     }
+    else
+    {
+        /* scroll not found, create one and add it at first position */
+        new_scroll = malloc (sizeof (*new_scroll));
+        if (new_scroll)
+        {
+            gui_window_scroll_init (new_scroll, buffer);
+            new_scroll->next_scroll = window->scroll;
+            (window->scroll)->prev_scroll = new_scroll;
+            window->scroll = new_scroll;
+        }
+    }
+
+    gui_window_scroll_remove_not_scrolled (window);
 }
 
 /*
@@ -510,12 +543,12 @@ gui_window_scroll_remove_buffer (struct t_gui_window *window,
 {
     struct t_gui_window_scroll *ptr_scroll;
 
-    if (window && buffer)
-    {
-        ptr_scroll = gui_window_scroll_search (window, buffer);
-        if (ptr_scroll)
-            gui_window_scroll_free (window, ptr_scroll);
-    }
+    if (!window || !buffer)
+        return;
+
+    ptr_scroll = gui_window_scroll_search (window, buffer);
+    if (ptr_scroll)
+        gui_window_scroll_free (window, ptr_scroll);
 }
 
 /*
@@ -643,7 +676,6 @@ gui_window_new (struct t_gui_window *parent_window, struct t_gui_buffer *buffer,
     /* coordinates */
     new_window->coords_size = 0;
     new_window->coords = NULL;
-    new_window->coords_x_message = 0;
 
     /* tree */
     new_window->ptr_tree = ptr_leaf;
@@ -651,7 +683,7 @@ gui_window_new (struct t_gui_window *parent_window, struct t_gui_buffer *buffer,
 
     /* add window to windows queue */
     new_window->prev_window = last_gui_window;
-    if (gui_windows)
+    if (last_gui_window)
         last_gui_window->next_window = new_window;
     else
         gui_windows = new_window;
@@ -736,37 +768,37 @@ gui_window_search_with_buffer (struct t_gui_buffer *buffer)
 int
 gui_window_get_integer (struct t_gui_window *window, const char *property)
 {
-    if (window && property)
-    {
-        if (string_strcasecmp (property, "number") == 0)
-            return window->number;
-        if (string_strcasecmp (property, "win_x") == 0)
-            return window->win_x;
-        if (string_strcasecmp (property, "win_y") == 0)
-            return window->win_y;
-        if (string_strcasecmp (property, "win_width") == 0)
-            return window->win_width;
-        if (string_strcasecmp (property, "win_height") == 0)
-            return window->win_height;
-        if (string_strcasecmp (property, "win_width_pct") == 0)
-            return window->win_width_pct;
-        if (string_strcasecmp (property, "win_height_pct") == 0)
-            return window->win_height_pct;
-        if (string_strcasecmp (property, "win_chat_x") == 0)
-            return window->win_chat_x;
-        if (string_strcasecmp (property, "win_chat_y") == 0)
-            return window->win_chat_y;
-        if (string_strcasecmp (property, "win_chat_width") == 0)
-            return window->win_chat_width;
-        if (string_strcasecmp (property, "win_chat_height") == 0)
-            return window->win_chat_height;
-        if (string_strcasecmp (property, "first_line_displayed") == 0)
-            return window->scroll->first_line_displayed;
-        if (string_strcasecmp (property, "scrolling") == 0)
-            return window->scroll->scrolling;
-        if (string_strcasecmp (property, "lines_after") == 0)
-            return window->scroll->lines_after;
-    }
+    if (!window || !property)
+        return 0;
+
+    if (string_strcasecmp (property, "number") == 0)
+        return window->number;
+    if (string_strcasecmp (property, "win_x") == 0)
+        return window->win_x;
+    if (string_strcasecmp (property, "win_y") == 0)
+        return window->win_y;
+    if (string_strcasecmp (property, "win_width") == 0)
+        return window->win_width;
+    if (string_strcasecmp (property, "win_height") == 0)
+        return window->win_height;
+    if (string_strcasecmp (property, "win_width_pct") == 0)
+        return window->win_width_pct;
+    if (string_strcasecmp (property, "win_height_pct") == 0)
+        return window->win_height_pct;
+    if (string_strcasecmp (property, "win_chat_x") == 0)
+        return window->win_chat_x;
+    if (string_strcasecmp (property, "win_chat_y") == 0)
+        return window->win_chat_y;
+    if (string_strcasecmp (property, "win_chat_width") == 0)
+        return window->win_chat_width;
+    if (string_strcasecmp (property, "win_chat_height") == 0)
+        return window->win_chat_height;
+    if (string_strcasecmp (property, "first_line_displayed") == 0)
+        return window->scroll->first_line_displayed;
+    if (string_strcasecmp (property, "scrolling") == 0)
+        return window->scroll->scrolling;
+    if (string_strcasecmp (property, "lines_after") == 0)
+        return window->scroll->lines_after;
 
     return 0;
 }
@@ -778,9 +810,8 @@ gui_window_get_integer (struct t_gui_window *window, const char *property)
 const char *
 gui_window_get_string (struct t_gui_window *window, const char *property)
 {
-    if (window && property)
-    {
-    }
+    if (!window || !property)
+        return NULL;
 
     return NULL;
 }
@@ -815,6 +846,9 @@ void
 gui_window_set_layout_plugin_name (struct t_gui_window *window,
                                    const char *plugin_name)
 {
+    if (!window)
+        return;
+
     if (window->layout_plugin_name)
     {
         free (window->layout_plugin_name);
@@ -833,6 +867,9 @@ void
 gui_window_set_layout_buffer_name (struct t_gui_window *window,
                                    const char *buffer_name)
 {
+    if (!window)
+        return;
+
     if (window->layout_buffer_name)
     {
         free (window->layout_buffer_name);
@@ -850,8 +887,11 @@ gui_window_set_layout_buffer_name (struct t_gui_window *window,
 void
 gui_window_coords_init_line (struct t_gui_window *window, int line)
 {
-    if (!window->coords || (line < 0) || (line >= window->coords_size))
+    if (!window || !window->coords || (line < 0)
+        || (line >= window->coords_size))
+    {
         return;
+    }
 
     window->coords[line].line = NULL;
     window->coords[line].data = NULL;
@@ -874,7 +914,7 @@ gui_window_coords_remove_line (struct t_gui_window *window,
 {
     int i;
 
-    if (!window->coords)
+    if (!window || !window->coords)
         return;
 
     for (i = 0; i < window->coords_size; i++)
@@ -895,7 +935,7 @@ gui_window_coords_remove_line_data (struct t_gui_window *window,
 {
     int i;
 
-    if (!window->coords)
+    if (!window || !window->coords)
         return;
 
     for (i = 0; i < window->coords_size; i++)
@@ -917,6 +957,9 @@ gui_window_coords_alloc (struct t_gui_window *window)
 {
     int i;
 
+    if (!window)
+        return;
+
     if (window->coords && (window->coords_size != window->win_chat_height))
     {
         free (window->coords);
@@ -932,7 +975,6 @@ gui_window_coords_alloc (struct t_gui_window *window)
             gui_window_coords_init_line (window, i);
         }
     }
-    window->coords_x_message = 0;
 }
 
 /*
@@ -945,6 +987,9 @@ gui_window_free (struct t_gui_window *window)
     struct t_gui_window *ptr_win, *old_current_window;
     int i;
 
+    if (!window)
+        return;
+
     old_current_window = gui_current_window;
 
     (void) hook_signal_send ("window_closing",
@@ -1019,7 +1064,7 @@ gui_window_free (struct t_gui_window *window)
 void
 gui_window_switch_previous (struct t_gui_window *window)
 {
-    if (!gui_init_ok)
+    if (!gui_init_ok || !window)
         return;
 
     gui_window_switch ((window->prev_window) ?
@@ -1033,7 +1078,7 @@ gui_window_switch_previous (struct t_gui_window *window)
 void
 gui_window_switch_next (struct t_gui_window *window)
 {
-    if (!gui_init_ok)
+    if (!gui_init_ok || !window)
         return;
 
     gui_window_switch ((window->next_window) ?
@@ -1066,7 +1111,7 @@ gui_window_switch_by_buffer (struct t_gui_window *window, int buffer_number)
 {
     struct t_gui_window *ptr_win;
 
-    if (!gui_init_ok)
+    if (!gui_init_ok || !window)
         return;
 
     ptr_win = (window->next_window) ? window->next_window : gui_windows;
@@ -1096,7 +1141,7 @@ gui_window_scroll (struct t_gui_window *window, char *scroll)
     struct t_gui_line *ptr_line;
     struct tm *date_tmp, line_date, old_line_date;
 
-    if (!window->buffer->lines->first_line)
+    if (!window || !window->buffer->lines->first_line)
         return;
 
     direction = 1;
@@ -1343,61 +1388,61 @@ gui_window_scroll_horiz (struct t_gui_window *window, char *scroll)
     char saved_char, *pos, *error;
     long number;
 
-    if (window->buffer->lines->first_line)
+    if (!window || !window->buffer->lines->first_line)
+        return;
+
+    direction = 1;
+    number = 0;
+    percentage = 0;
+
+    /* search direction */
+    if (scroll[0] == '-')
     {
-        direction = 1;
-        number = 0;
-        percentage = 0;
+        direction = -1;
+        scroll++;
+    }
+    else if (scroll[0] == '+')
+    {
+        direction = +1;
+        scroll++;
+    }
 
-        /* search direction */
-        if (scroll[0] == '-')
-        {
-            direction = -1;
-            scroll++;
-        }
-        else if (scroll[0] == '+')
-        {
-            direction = +1;
-            scroll++;
-        }
+    /* search number and percentage */
+    pos = scroll;
+    while (pos && pos[0] && isdigit ((unsigned char)pos[0]))
+    {
+        pos++;
+    }
+    if (pos && (pos > scroll))
+    {
+        percentage = (pos[0] == '%') ? 1 : 0;
+        saved_char = pos[0];
+        pos[0] = '\0';
+        error = NULL;
+        number = strtol (scroll, &error, 10);
+        if (!error || error[0])
+            number = 0;
+        pos[0] = saved_char;
+    }
 
-        /* search number and percentage */
-        pos = scroll;
-        while (pos && pos[0] && isdigit ((unsigned char)pos[0]))
-        {
-            pos++;
-        }
-        if (pos && (pos > scroll))
-        {
-            percentage = (pos[0] == '%') ? 1 : 0;
-            saved_char = pos[0];
-            pos[0] = '\0';
-            error = NULL;
-            number = strtol (scroll, &error, 10);
-            if (!error || error[0])
-                number = 0;
-            pos[0] = saved_char;
-        }
+    /* for percentage, compute number of columns */
+    if (percentage)
+    {
+        number = (window->win_chat_width * number) / 100;
+    }
 
-        /* for percentage, compute number of columns */
-        if (percentage)
-        {
-            number = (window->win_chat_width * number) / 100;
-        }
+    /* number must be different from 0 */
+    if (number == 0)
+        return;
 
-        /* number must be different from 0 */
-        if (number == 0)
-            return;
-
-        /* do the horizontal scroll! */
-        start_col = window->scroll->start_col + (number * direction);
-        if (start_col < 0)
-            start_col = 0;
-        if (start_col != window->scroll->start_col)
-        {
-            window->scroll->start_col = start_col;
-            gui_buffer_ask_chat_refresh (window->buffer, 2);
-        }
+    /* do the horizontal scroll! */
+    start_col = window->scroll->start_col + (number * direction);
+    if (start_col < 0)
+        start_col = 0;
+    if (start_col != window->scroll->start_col)
+    {
+        window->scroll->start_col = start_col;
+        gui_buffer_ask_chat_refresh (window->buffer, 2);
     }
 }
 
@@ -1410,6 +1455,9 @@ gui_window_scroll_previous_highlight (struct t_gui_window *window)
 {
     struct t_gui_line *ptr_line;
 
+    if (!window)
+        return;
+
     if ((window->buffer->type == GUI_BUFFER_TYPE_FORMATTED)
         && (window->buffer->text_search == GUI_TEXT_SEARCH_DISABLED))
     {
@@ -1445,6 +1493,9 @@ gui_window_scroll_next_highlight (struct t_gui_window *window)
 {
     struct t_gui_line *ptr_line;
 
+    if (!window)
+        return;
+
     if ((window->buffer->type == GUI_BUFFER_TYPE_FORMATTED)
         && (window->buffer->text_search == GUI_TEXT_SEARCH_DISABLED))
     {
@@ -1478,6 +1529,9 @@ gui_window_scroll_next_highlight (struct t_gui_window *window)
 void
 gui_window_scroll_unread (struct t_gui_window *window)
 {
+    if (!window)
+        return;
+
     if (window->buffer->text_search == GUI_TEXT_SEARCH_DISABLED)
     {
         if (CONFIG_STRING(config_look_read_marker) &&
@@ -1517,6 +1571,9 @@ gui_window_search_text (struct t_gui_window *window)
 {
     struct t_gui_line *ptr_line;
 
+    if (!window)
+        return 0;
+
     if (window->buffer->text_search == GUI_TEXT_SEARCH_BACKWARD)
     {
         if (window->buffer->lines->first_line
@@ -1567,12 +1624,18 @@ gui_window_search_text (struct t_gui_window *window)
 }
 
 /*
- * Starts search in a buffer.
+ * Starts search in a buffer at a given position
+ * (or in whole buffer if text_search_start_line is NULL).
  */
 
 void
-gui_window_search_start (struct t_gui_window *window)
+gui_window_search_start (struct t_gui_window *window,
+                         struct t_gui_line *text_search_start_line)
 {
+    if (!window)
+        return;
+
+    window->scroll->text_search_start_line = text_search_start_line;
     window->buffer->text_search =
         (window->buffer->type == GUI_BUFFER_TYPE_FORMATTED) ?
         GUI_TEXT_SEARCH_BACKWARD : GUI_TEXT_SEARCH_FORWARD;
@@ -1625,7 +1688,10 @@ gui_window_search_start (struct t_gui_window *window)
 void
 gui_window_search_restart (struct t_gui_window *window)
 {
-    window->scroll->start_line = NULL;
+    if (!window)
+        return;
+
+    window->scroll->start_line = window->scroll->text_search_start_line;
     window->scroll->start_line_pos = 0;
     window->buffer->text_search =
         (window->buffer->type == GUI_BUFFER_TYPE_FORMATTED) ?
@@ -1639,19 +1705,23 @@ gui_window_search_restart (struct t_gui_window *window)
         if (CONFIG_BOOLEAN(config_look_search_text_not_found_alert)
             && window->buffer->input_buffer && window->buffer->input_buffer[0])
         {
-            printf ("\a");
+            fprintf (stderr, "\a");
+            fflush (stderr);
         }
         gui_buffer_ask_chat_refresh (window->buffer, 2);
     }
 }
 
 /*
- * Stops search in a buffer.
+ * Ends search mode in a buffer (helper function).
  */
 
 void
-gui_window_search_stop (struct t_gui_window *window)
+gui_window_search_end (struct t_gui_window *window)
 {
+    if (!window)
+        return;
+
     window->buffer->text_search = GUI_TEXT_SEARCH_DISABLED;
     window->buffer->text_search = 0;
     if (window->buffer->text_search_regex_compiled)
@@ -1671,8 +1741,37 @@ gui_window_search_stop (struct t_gui_window *window)
         free (window->buffer->text_search_input);
         window->buffer->text_search_input = NULL;
     }
-    window->scroll->start_line = NULL;
+}
+
+/*
+ * Stops search in a buffer at current position.
+ */
+
+void
+gui_window_search_stop_here (struct t_gui_window *window)
+{
+    if (!window)
+        return;
+
+    gui_window_search_end (window);
+    window->scroll->text_search_start_line = NULL;
+    gui_buffer_ask_chat_refresh (window->buffer, 2);
+}
+
+/*
+ * Stops search in a buffer.
+ */
+
+void
+gui_window_search_stop (struct t_gui_window *window)
+{
+    if (!window)
+        return;
+
+    gui_window_search_end (window);
+    window->scroll->start_line = window->scroll->text_search_start_line;
     window->scroll->start_line_pos = 0;
+    window->scroll->text_search_start_line = NULL;
     gui_hotlist_remove_buffer (window->buffer, 0);
     gui_buffer_ask_chat_refresh (window->buffer, 2);
 }
@@ -1686,7 +1785,7 @@ gui_window_zoom (struct t_gui_window *window)
 {
     struct t_gui_layout *ptr_layout;
 
-    if (!gui_init_ok)
+    if (!gui_init_ok || !window)
         return;
 
     ptr_layout = gui_layout_search (GUI_LAYOUT_ZOOM);
@@ -1727,11 +1826,13 @@ gui_window_zoom (struct t_gui_window *window)
  */
 
 struct t_hdata *
-gui_window_hdata_window_cb (void *data, const char *hdata_name)
+gui_window_hdata_window_cb (const void *pointer, void *data,
+                            const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = hdata_new (NULL, hdata_name, "prev_window", "next_window",
@@ -1774,11 +1875,13 @@ gui_window_hdata_window_cb (void *data, const char *hdata_name)
  */
 
 struct t_hdata *
-gui_window_hdata_window_scroll_cb (void *data, const char *hdata_name)
+gui_window_hdata_window_scroll_cb (const void *pointer, void *data,
+                                   const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = hdata_new (NULL, hdata_name, "prev_scroll", "next_scroll",
@@ -1792,6 +1895,7 @@ gui_window_hdata_window_scroll_cb (void *data, const char *hdata_name)
         HDATA_VAR(struct t_gui_window_scroll, scrolling, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_window_scroll, start_col, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_window_scroll, lines_after, INTEGER, 0, NULL, NULL);
+        HDATA_VAR(struct t_gui_window_scroll, text_search_start_line, POINTER, 0, NULL, "line");
         HDATA_VAR(struct t_gui_window_scroll, prev_scroll, POINTER, 0, NULL, hdata_name);
         HDATA_VAR(struct t_gui_window_scroll, next_scroll, POINTER, 0, NULL, hdata_name);
     }
@@ -1803,11 +1907,13 @@ gui_window_hdata_window_scroll_cb (void *data, const char *hdata_name)
  */
 
 struct t_hdata *
-gui_window_hdata_window_tree_cb (void *data, const char *hdata_name)
+gui_window_hdata_window_tree_cb (const void *pointer, void *data,
+                                 const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = hdata_new (NULL, hdata_name, NULL, NULL, 0, 0, NULL, NULL);
@@ -1926,7 +2032,6 @@ gui_window_print_log ()
         log_printf ("  scroll. . . . . . . : 0x%lx", ptr_window->scroll);
         log_printf ("  coords_size . . . . : %d",    ptr_window->coords_size);
         log_printf ("  coords. . . . . . . : 0x%lx", ptr_window->coords);
-        log_printf ("  coords_x_message. . : %d",    ptr_window->coords_x_message);
         log_printf ("  ptr_tree. . . . . . : 0x%lx", ptr_window->ptr_tree);
         log_printf ("  prev_window . . . . : 0x%lx", ptr_window->prev_window);
         log_printf ("  next_window . . . . : 0x%lx", ptr_window->next_window);
@@ -1936,15 +2041,16 @@ gui_window_print_log ()
         {
             log_printf ("");
             log_printf ("  [scroll (addr:0x%lx)]", ptr_scroll);
-            log_printf ("    buffer. . . . . . . : 0x%lx", ptr_scroll->buffer);
-            log_printf ("    first_line_displayed: %d",    ptr_scroll->first_line_displayed);
-            log_printf ("    start_line. . . . . : 0x%lx", ptr_scroll->start_line);
-            log_printf ("    start_line_pos. . . : %d",    ptr_scroll->start_line_pos);
-            log_printf ("    scrolling . . . . . : %d",    ptr_scroll->scrolling);
-            log_printf ("    start_col . . . . . : %d",    ptr_scroll->start_col);
-            log_printf ("    lines_after . . . . : %d",    ptr_scroll->lines_after);
-            log_printf ("    prev_scroll . . . . : 0x%lx", ptr_scroll->prev_scroll);
-            log_printf ("    next_scroll . . . . : 0x%lx", ptr_scroll->next_scroll);
+            log_printf ("    buffer. . . . . . . . : 0x%lx", ptr_scroll->buffer);
+            log_printf ("    first_line_displayed. : %d",    ptr_scroll->first_line_displayed);
+            log_printf ("    start_line. . . . . . : 0x%lx", ptr_scroll->start_line);
+            log_printf ("    start_line_pos. . . . : %d",    ptr_scroll->start_line_pos);
+            log_printf ("    scrolling . . . . . . : %d",    ptr_scroll->scrolling);
+            log_printf ("    start_col . . . . . . : %d",    ptr_scroll->start_col);
+            log_printf ("    lines_after . . . . . : %d",    ptr_scroll->lines_after);
+            log_printf ("    text_search_start_line: 0x%lx", ptr_scroll->text_search_start_line);
+            log_printf ("    prev_scroll . . . . . : 0x%lx", ptr_scroll->prev_scroll);
+            log_printf ("    next_scroll . . . . . : 0x%lx", ptr_scroll->next_scroll);
         }
 
         for (ptr_bar_win = ptr_window->bar_windows; ptr_bar_win;
