@@ -1,7 +1,7 @@
 /*
  * fset-option.c - manage options displayed by Fast Set buffer
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -313,13 +313,14 @@ fset_option_match_filter (struct t_fset_option *fset_option, const char *filter)
 
     if (strncmp (filter, "c:", 2) == 0)
     {
+        /* filter by evaluated condition */
         weechat_hashtable_set (fset_option_filter_hashtable_pointers,
                                "fset_option", fset_option);
         fset_option_add_option_in_hashtable (
             fset_option_filter_hashtable_extra_vars,
             fset_option);
         result = weechat_string_eval_expression (
-            fset_option_filter + 2,
+            filter + 2,
             fset_option_filter_hashtable_pointers,
             fset_option_filter_hashtable_extra_vars,
             fset_option_filter_hashtable_options);
@@ -372,6 +373,22 @@ fset_option_match_filter (struct t_fset_option *fset_option, const char *filter)
         /* filter by modified values */
         return (fset_option_value_is_changed (fset_option)) ? 1 : 0;
     }
+    else if (strncmp (filter, "h=", 2) == 0)
+    {
+        /* filter by help text (translated) */
+        return (fset_option_string_match (
+                    (fset_option->description && fset_option->description[0]) ?
+                    _(fset_option->description) : "",
+                    filter + 2)) ? 1 : 0;
+    }
+    else if (strncmp (filter, "he=", 3) == 0)
+    {
+        /* filter by help text (in English) */
+        return (fset_option_string_match (
+                    (fset_option->description && fset_option->description[0]) ?
+                    fset_option->description : "",
+                    filter + 3)) ? 1 : 0;
+    }
     else if (strncmp (filter, "==", 2) == 0)
     {
         /* filter by exact value */
@@ -1165,8 +1182,9 @@ fset_option_set (struct t_fset_option *fset_option,
                  struct t_gui_buffer *buffer,
                  int set_mode)
 {
-    int use_mute, add_quotes, input_pos;
-    char str_input[4096], str_pos[32];
+    int use_mute, add_quotes, length_input, input_pos;
+    char *ptr_value, *str_input, str_pos[32];
+    char empty_value[1] = { '\0' };
 
     /* make C compiler happy */
     (void) option;
@@ -1174,14 +1192,24 @@ fset_option_set (struct t_fset_option *fset_option,
     if (!fset_option)
         return;
 
+    if (set_mode == -1)
+        ptr_value = empty_value;
+    else
+        ptr_value = (fset_option->value) ? fset_option->value : empty_value;
+
+    length_input = 64 + strlen (fset_option->name) + strlen (ptr_value) + 1;
+    str_input = malloc (length_input);
+    if (!str_input)
+        return;
+
     use_mute = weechat_config_boolean (fset_config_look_use_mute);
     add_quotes = (fset_option->type == FSET_OPTION_TYPE_STRING) ? 1 : 0;
-    snprintf (str_input, sizeof (str_input),
+    snprintf (str_input, length_input,
               "%s/set %s %s%s%s",
               (use_mute) ? "/mute " : "",
               fset_option->name,
               (add_quotes) ? "\"" : "",
-              (set_mode != -1) ? ((fset_option->value) ? fset_option->value : "") : "",
+              ptr_value,
               (add_quotes) ? "\"" : "");
     weechat_buffer_set (buffer, "input", str_input);
     input_pos = ((use_mute) ? 6 : 0) +  /* "/mute " */
@@ -1192,6 +1220,8 @@ fset_option_set (struct t_fset_option *fset_option,
                             weechat_utf8_strlen (fset_option->value) : 0) : 0);
     snprintf (str_pos, sizeof (str_pos), "%d", input_pos);
     weechat_buffer_set (buffer, "input_pos", str_pos);
+
+    free (str_input);
 }
 
 /*
@@ -1417,6 +1447,7 @@ fset_option_config_changed (const char *option_name)
             /* option added: get options and refresh the whole buffer */
             full_refresh = 1;
         }
+        fset_option_free (new_fset_option);
     }
 
     if (full_refresh)
@@ -1432,6 +1463,7 @@ fset_option_config_changed (const char *option_name)
             ptr_fset_option = weechat_arraylist_get (fset_options, line);
             if (ptr_fset_option
                 && ptr_fset_option->parent_name
+                && option_name
                 && (strcmp (ptr_fset_option->parent_name, option_name) == 0))
             {
                 ptr_option = weechat_config_get (ptr_fset_option->name);
@@ -1476,9 +1508,20 @@ fset_option_config_timer_cb (const void *pointer,
     (void) data;
     (void) remaining_calls;
 
-    weechat_hashtable_map (fset_option_timer_options_changed,
-                           &fset_option_timer_option_changed_cb,
-                           NULL);
+    if (weechat_hashtable_get_integer (
+            fset_option_timer_options_changed,
+            "items_count") >= FSET_OPTION_TIMER_MAX_OPTIONS_CHANGED)
+    {
+        fset_option_get_options ();
+        fset_buffer_refresh (1);
+    }
+    else
+    {
+        weechat_hashtable_map (fset_option_timer_options_changed,
+                               &fset_option_timer_option_changed_cb,
+                               NULL);
+    }
+
     weechat_hashtable_remove_all (fset_option_timer_options_changed);
 
     fset_option_timer_hook = NULL;
@@ -1496,7 +1539,7 @@ fset_option_config_cb (const void *pointer,
                        const char *option,
                        const char *value)
 {
-    const char *ptr_info;
+    char *info;
 
     /* make C compiler happy */
     (void) pointer;
@@ -1507,13 +1550,34 @@ fset_option_config_cb (const void *pointer,
     if (!fset_buffer)
         return WEECHAT_RC_OK;
 
-    /* do nothing if WeeChat is upgrading */
-    ptr_info = weechat_info_get ("weechat_upgrading", NULL);
-    if (ptr_info && (strcmp (ptr_info, "1") == 0))
+    /* do nothing if auto-refresh is disabled for this option */
+    if (!weechat_string_match_list (option,
+                                    (const char **)fset_config_auto_refresh,
+                                    0))
         return WEECHAT_RC_OK;
 
-    weechat_hashtable_set (fset_option_timer_options_changed,
-                           option, NULL);
+    /* do nothing if WeeChat is upgrading */
+    info = weechat_info_get ("weechat_upgrading", NULL);
+    if (info && (strcmp (info, "1") == 0))
+    {
+        free (info);
+        return WEECHAT_RC_OK;
+    }
+    if (info)
+        free (info);
+
+    /*
+     * we limit the number of options to display with the timer; for example
+     * on /reload, many options are changed, so we'll get all options and
+     * display them, instead of change them one by one, which is very slow
+     */
+    if (weechat_hashtable_get_integer (
+            fset_option_timer_options_changed,
+            "items_count") < FSET_OPTION_TIMER_MAX_OPTIONS_CHANGED)
+    {
+        weechat_hashtable_set (fset_option_timer_options_changed,
+                               option, NULL);
+    }
 
     if (!fset_option_timer_hook)
     {
