@@ -26,15 +26,15 @@
     int                                                                 \
     irc_protocol_cb_##__command (struct t_irc_server *server,           \
                                  time_t date,                           \
+                                 const char *irc_message,               \
                                  struct t_hashtable *tags,              \
                                  const char *nick,                      \
                                  const char *address,                   \
                                  const char *host,                      \
                                  const char *command,                   \
                                  int ignored,                           \
-                                 int argc,                              \
-                                 char **argv,                           \
-                                 char **argv_eol)
+                                 const char **params,                   \
+                                 int num_params)
 #define IRCB(__message, __decode_color, __keep_trailing_spaces,         \
              __func_cb)                                                 \
     { #__message,                                                       \
@@ -42,33 +42,34 @@
       __keep_trailing_spaces,                                           \
       &irc_protocol_cb_##__func_cb }
 
-#define IRC_PROTOCOL_MIN_ARGS(__min_args)                               \
+#define IRC_PROTOCOL_MIN_PARAMS(__min_params)                           \
     (void) date;                                                        \
+    (void) irc_message;                                                 \
     (void) tags;                                                        \
     (void) nick;                                                        \
     (void) address;                                                     \
     (void) host;                                                        \
     (void) command;                                                     \
     (void) ignored;                                                     \
-    (void) argv;                                                        \
-    (void) argv_eol;                                                    \
-    if (argc < __min_args)                                              \
+    (void) params;                                                      \
+    (void) num_params;                                                  \
+    if (num_params < __min_params)                                      \
     {                                                                   \
         weechat_printf (server->buffer,                                 \
-                        _("%s%s: too few arguments received from IRC "  \
-                          "server for command \"%s\" (received: %d "    \
-                          "arguments, expected: at least %d)"),         \
+                        _("%s%s: too few parameters received in "       \
+                          "command \"%s\" (received: %d, expected: "    \
+                          "at least %d)"),                              \
                         weechat_prefix ("error"), IRC_PLUGIN_NAME,      \
-                        command, argc, __min_args);                     \
+                        command, num_params, __min_params);             \
         return WEECHAT_RC_ERROR;                                        \
     }
 
-#define IRC_PROTOCOL_CHECK_HOST                                         \
-    if (argv[0][0] != ':')                                              \
+#define IRC_PROTOCOL_CHECK_NICK                                         \
+    if (!nick || !nick[0])                                              \
     {                                                                   \
         weechat_printf (server->buffer,                                 \
-                        _("%s%s: \"%s\" command received without "      \
-                          "host"),                                      \
+                        _("%s%s: command \"%s\" received without "      \
+                          "nick"),                                      \
                         weechat_prefix ("error"), IRC_PLUGIN_NAME,      \
                         command);                                       \
         return WEECHAT_RC_ERROR;                                        \
@@ -77,11 +78,12 @@
 struct t_irc_server;
 
 typedef int (t_irc_recv_func)(struct t_irc_server *server,
-                              time_t date, struct t_hashtable *tags,
+                              time_t date, const char *irc_message,
+                              struct t_hashtable *tags,
                               const char *nick, const char *address,
                               const char *host, const char *command,
                               int ignored,
-                              int argc, char **argv, char **argv_eol);
+                              const char **params, int num_params);
 
 struct t_irc_protocol_msg
 {
