@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2012 Simon Arlott
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_PLUGIN_SCRIPT_API_H
@@ -25,6 +25,10 @@
 
 extern void plugin_script_api_charset_set (struct t_plugin_script *script,
                                            const char *charset);
+extern int plugin_script_api_string_match_list (struct t_weechat_plugin *weechat_plugin,
+                                                const char *string,
+                                                const char *masks,
+                                                int case_sensitive);
 extern struct t_config_file *plugin_script_api_config_new (struct t_weechat_plugin *weechat_plugin,
                                                            struct t_plugin_script *script,
                                                            const char *name,
@@ -203,6 +207,16 @@ extern struct t_hook *plugin_script_api_hook_connect (struct t_weechat_plugin *w
                                                                       const char *ip_address),
                                                       const char *function,
                                                       const char *data);
+extern struct t_hook *plugin_script_api_hook_line (struct t_weechat_plugin *weechat_plugin,
+                                                   struct t_plugin_script *script,
+                                                   const char *buffer_type,
+                                                   const char *buffer_name,
+                                                   const char *tags,
+                                                   struct t_hashtable *(*callback)(const void *pointer,
+                                                                                   void *data,
+                                                                                   struct t_hashtable *line),
+                                                   const char *function,
+                                                   const char *data);
 extern struct t_hook *plugin_script_api_hook_print (struct t_weechat_plugin *weechat_plugin,
                                                     struct t_plugin_script *script,
                                                     struct t_gui_buffer *buffer,
@@ -275,10 +289,10 @@ extern struct t_hook *plugin_script_api_hook_info (struct t_weechat_plugin *weec
                                                    const char *info_name,
                                                    const char *description,
                                                    const char *args_description,
-                                                   const char *(*callback)(const void *pointer,
-                                                                           void *data,
-                                                                           const char *info_name,
-                                                                           const char *arguments),
+                                                   char *(*callback)(const void *pointer,
+                                                                     void *data,
+                                                                     const char *info_name,
+                                                                     const char *arguments),
                                                    const char *function,
                                                    const char *data);
 extern struct t_hook *plugin_script_api_hook_info_hashtable (struct t_weechat_plugin *weechat_plugin,
@@ -339,6 +353,11 @@ extern struct t_gui_bar_item *plugin_script_api_bar_item_new (struct t_weechat_p
                                                                                       struct t_hashtable *extra_info),
                                                               const char *function,
                                                               const char *data);
+extern int plugin_script_api_command_options (struct t_weechat_plugin *weechat_plugin,
+                                              struct t_plugin_script *script,
+                                              struct t_gui_buffer *buffer,
+                                              const char *command,
+                                              struct t_hashtable *options);
 extern int plugin_script_api_command (struct t_weechat_plugin *weechat_plugin,
                                       struct t_plugin_script *script,
                                       struct t_gui_buffer *buffer,
