@@ -1,7 +1,7 @@
 /*
  * script-mouse.c - mouse actions for script
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -39,7 +39,7 @@ script_mouse_focus_chat_cb (const void *pointer, void *data,
 {
     const char *buffer;
     int rc;
-    long unsigned int value;
+    unsigned long value;
     struct t_gui_buffer *ptr_buffer;
     long x;
     char *error, str_date[64];
@@ -95,7 +95,7 @@ script_mouse_focus_chat_cb (const void *pointer, void *data,
     weechat_hashtable_set (info, "script_requirements", ptr_script->requirements);
     weechat_hashtable_set (info, "script_min_weechat", ptr_script->min_weechat);
     weechat_hashtable_set (info, "script_max_weechat", ptr_script->max_weechat);
-    weechat_hashtable_set (info, "script_md5sum", ptr_script->md5sum);
+    weechat_hashtable_set (info, "script_sha512sum", ptr_script->sha512sum);
     weechat_hashtable_set (info, "script_url", ptr_script->url);
     tm = localtime (&ptr_script->date_added);
     if (strftime (str_date, sizeof (str_date), "%Y-%m-%d %H:%M:%S", tm) == 0)
