@@ -1,7 +1,7 @@
 /*
  * trigger-command.c - trigger command
  *
- * Copyright (C) 2014-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,11 +16,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <stdarg.h>
 #include <string.h>
 
 #include "../weechat-plugin.h"
@@ -476,6 +477,19 @@ end:
         free (name2);
 }
 
+/*
+ * Builds a string with the command to create the trigger.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+trigger_command_build_string (const char *format, ...)
+{
+    weechat_va_format (format);
+    return vbuffer;
+}
+
 /*
  * Callback for command "/trigger": manage triggers.
  */
@@ -487,7 +501,7 @@ trigger_command_trigger (const void *pointer, void *data,
 {
     struct t_trigger *ptr_trigger, *ptr_trigger2;
     struct t_trigger_regex *regex;
-    char *value, **sargv, **items, input[1024], str_pos[16];
+    char *value, **sargv, **items, *input, str_pos[16];
     int rc, i, j, type, count, index_option, enable, sargc, num_items, add_rc;
     int regex_count, regex_rc;
 
@@ -682,20 +696,30 @@ trigger_command_trigger (const void *pointer, void *data,
                 goto end;
             }
         }
-        items = weechat_string_split (trigger_hook_default_rc[type], ",", 0, 0,
+        items = weechat_string_split (trigger_hook_default_rc[type],
+                                      ",",
+                                      NULL,
+                                      WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                      | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                      | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                      0,
                                       &num_items);
-        snprintf (input, sizeof (input),
-                  "/trigger add name %s \"%s\" \"%s\" \"%s\" \"%s\"%s%s%s",
-                  trigger_hook_type_string[type],
-                  trigger_hook_default_arguments[type],
-                  TRIGGER_HOOK_DEFAULT_CONDITIONS,
-                  TRIGGER_HOOK_DEFAULT_REGEX,
-                  TRIGGER_HOOK_DEFAULT_COMMAND,
-                  (items && (num_items > 0)) ? " \"" : "",
-                  (items && (num_items > 0)) ? items[0] : "",
-                  (items && (num_items > 0)) ? "\"" : "");
-        weechat_buffer_set (buffer, "input", input);
-        weechat_buffer_set (buffer, "input_pos", "13");
+        input = trigger_command_build_string (
+            "/trigger add name %s \"%s\" \"%s\" \"%s\" \"%s\"%s%s%s",
+            trigger_hook_type_string[type],
+            trigger_hook_default_arguments[type],
+            TRIGGER_HOOK_DEFAULT_CONDITIONS,
+            TRIGGER_HOOK_DEFAULT_REGEX,
+            TRIGGER_HOOK_DEFAULT_COMMAND,
+            (items && (num_items > 0)) ? " \"" : "",
+            (items && (num_items > 0)) ? items[0] : "",
+            (items && (num_items > 0)) ? "\"" : "");
+        if (input)
+        {
+            weechat_buffer_set (buffer, "input", input);
+            weechat_buffer_set (buffer, "input_pos", "13");
+            free (input);
+        }
         goto end;
     }
 
@@ -722,27 +746,31 @@ trigger_command_trigger (const void *pointer, void *data,
             goto end;
         }
         add_rc = trigger_hook_default_rc[weechat_config_integer (ptr_trigger->options[TRIGGER_OPTION_HOOK])][0];
-        snprintf (input, sizeof (input),
-                  "//trigger %s %s %s \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
-                  (weechat_strcasecmp (argv[1], "recreate") == 0) ? "addreplace" : "add",
-                  ptr_trigger->name,
-                  weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_HOOK]),
-                  weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_ARGUMENTS]),
-                  weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_CONDITIONS]),
-                  weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_REGEX]),
-                  weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_COMMAND]),
-                  (add_rc) ? weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_RETURN_CODE]) : "",
-                  weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_POST_ACTION]));
-        if (weechat_strcasecmp (argv[1], "output") == 0)
+        input = trigger_command_build_string (
+            "//trigger %s %s %s \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
+            (weechat_strcasecmp (argv[1], "recreate") == 0) ? "addreplace" : "add",
+            ptr_trigger->name,
+            weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_HOOK]),
+            weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_ARGUMENTS]),
+            weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_CONDITIONS]),
+            weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_REGEX]),
+            weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_COMMAND]),
+            (add_rc) ? weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_RETURN_CODE]) : "",
+            weechat_config_string (ptr_trigger->options[TRIGGER_OPTION_POST_ACTION]));
+        if (input)
         {
-            weechat_command (buffer, input);
-        }
-        else
-        {
-            weechat_buffer_set (buffer, "input", input + 1);
-            snprintf (str_pos, sizeof (str_pos),
-                      "%d", weechat_utf8_strlen (input + 1));
-            weechat_buffer_set (buffer, "input_pos", str_pos);
+            if (weechat_strcasecmp (argv[1], "output") == 0)
+            {
+                weechat_command (buffer, input);
+            }
+            else
+            {
+                weechat_buffer_set (buffer, "input", input + 1);
+                snprintf (str_pos, sizeof (str_pos),
+                          "%d", weechat_utf8_strlen (input + 1));
+                weechat_buffer_set (buffer, "input_pos", str_pos);
+            }
+            free (input);
         }
         goto end;
     }
@@ -1153,20 +1181,26 @@ trigger_command_init ()
            "     addoff: add a trigger (disabled)\n"
            " addreplace: add or replace an existing trigger\n"
            "       name: name of trigger\n"
-           "       hook: signal, hsignal, modifier, print, command, command_run, "
-           "timer, config, focus\n"
+           "       hook: signal, hsignal, modifier, line, print, command, "
+           "command_run, timer, config, focus, info, info_hashtable\n"
            "  arguments: arguments for the hook, depending on hook (separated "
            "by semicolons):\n"
            "             signal: name(s) of signal (required)\n"
            "             hsignal: name(s) of hsignal (required)\n"
            "             modifier: name(s) of modifier (required)\n"
+           "             line: buffer type (\"formatted\", \"free\" or \"*\"), "
+           "list of buffer masks, tags\n"
            "             print: buffer, tags, message, strip colors\n"
            "             command: command (required), description, arguments, "
-           "description of arguments, completion\n"
+           "description of arguments, completion (all arguments except command "
+           "are evaluated, \"${tg_trigger_name}\" is replaced by the trigger "
+           "name, see /help eval)\n"
            "             command_run: command(s) (required)\n"
            "             timer: interval (required), align on second, max calls\n"
            "             config: name(s) of option (required)\n"
            "             focus: name(s) of area (required)\n"
+           "             info: name(s) of info (required)\n"
+           "             info_hashtable: name(s) of info (required)\n"
            " conditions: evaluated conditions for the trigger\n"
            "      regex: one or more regular expressions to replace strings "
            "in variables\n"
@@ -1212,7 +1246,8 @@ trigger_command_init ()
            "  2. replace text using POSIX extended regular expression(s) (if "
            "defined in trigger)\n"
            "  3. execute command(s) (if defined in trigger)\n"
-           "  4. exit with a return code (except for modifiers and focus)\n"
+           "  4. exit with a return code (except for modifier, line, focus, "
+           "info and info_hashtable)\n"
            "  5. perform post action\n"
            "\n"
            "Examples (you can also look at default triggers with /trigger "
@@ -1220,9 +1255,9 @@ trigger_command_init ()
            "  add text attributes *bold*, _underline_ and /italic/ (only in "
            "user messages):\n"
            "    /trigger add effects modifier weechat_print \"${tg_tag_nick}\" "
-           "\"==\\*(\\S+)\\*==*${color:bold}${re:1}${color:-bold}*== "
-           "==_(\\S+)_==_${color:underline}${re:1}${color:-underline}_== "
-           "==/(\\S+)/==/${color:italic}${re:1}${color:-italic}/\"\n"
+           "\"==\\*([^ ]+)\\*==*${color:bold}${re:1}${color:-bold}*== "
+           "==_([^ ]+)_==_${color:underline}${re:1}${color:-underline}_== "
+           "==/([^ ]+)/==/${color:italic}${re:1}${color:-italic}/\"\n"
            "  hide nicklist bar on small terminals:\n"
            "    /trigger add resize_small signal signal_sigwinch "
            "\"${info:term_width} < 100\" \"\" \"/bar hide nicklist\"\n"
