@@ -183,7 +183,7 @@ TEST(Scripts, API)
     /* check that there was no errors in script generation */
     LONGS_EQUAL(0, api_tests_errors);
 
-    /* unload generator scritp */
+    /* unload generator script */
     snprintf (str_command, sizeof (str_command),
               "/script unload testapigen.py");
     run_cmd (str_command);
@@ -198,7 +198,7 @@ TEST(Scripts, API)
         snprintf (str_condition, sizeof (str_condition),
                   "${plugin.name} == %s",
                   languages[i][0]);
-        if (!hdata_search (hdata, plugins, str_condition, 1))
+        if (!hdata_search (hdata, plugins, str_condition, NULL, NULL, NULL, 1))
             continue;
 
         /*
