@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
+# Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
 #
 # This file is part of WeeChat, the extensible chat client.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+# along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
 #
 
 #
@@ -33,15 +33,24 @@
 #             defaults to current directory
 #
 
+# exit on any error
+set -e
+
+error ()
+{
+    echo >&2 "ERROR: $*"
+    exit 1
+}
+
 # check git repository
 ROOT_DIR=$(git rev-parse --show-toplevel)
-if [ -z "${ROOT_DIR}" -o ! -d "${ROOT_DIR}/.git" ]; then
-    echo "This script must be run from WeeChat git repository."
-    exit 1
+if [ -z "${ROOT_DIR}" ] || [ ! -d "${ROOT_DIR}/.git" ]; then
+    error "this script must be run from WeeChat git repository."
 fi
+cd "${ROOT_DIR}"
 
 # default values
-VERSION="$(${ROOT_DIR}/version.sh devel-full)"
+VERSION="$("${ROOT_DIR}/version.sh" devel-full)"
 TREEISH="HEAD"
 OUTPATH="$(pwd)"
 
@@ -55,16 +64,17 @@ if [ $# -ge 3 ]; then
     OUTPATH=$(cd "$3"; pwd)
 fi
 
-cd "${ROOT_DIR}"
-
 PREFIX="weechat-${VERSION}/"
 FILE="${OUTPATH}/weechat-${VERSION}.tar"
 
 echo "Building package ${FILE}.gz"
-git archive --prefix=${PREFIX} ${TREEISH} | gzip -c >${FILE}.gz
+git archive --prefix="${PREFIX}" "${TREEISH}" | gzip -c >"${FILE}.gz"
 
 echo "Building package ${FILE}.bz2"
-git archive --prefix=${PREFIX} ${TREEISH} | bzip2 -c >${FILE}.bz2
+git archive --prefix="${PREFIX}" "${TREEISH}" | bzip2 -c >"${FILE}.bz2"
 
 echo "Building package ${FILE}.xz"
-git archive --prefix=${PREFIX} ${TREEISH} | xz -c >${FILE}.xz
+git archive --prefix="${PREFIX}" "${TREEISH}" | xz -c >"${FILE}.xz"
+
+echo "Building package ${FILE}.zst"
+git archive --prefix="${PREFIX}" "${TREEISH}" | zstd -c -15 >"${FILE}.zst"
